

/*	Copyright (C) 2005, 2006 by Andy Grebe a.k.a. tecknophreak(tecknophreak@gmail.com)
 *
 *	This file is part of TouchMe Games.
 *
 *	TouchMe Games is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	TouchMe Games is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TouchMe Games; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <dartcountdown.h>
#include <algorithm>
#include <sstream>
#include <iomanip>


GtkWidget *DartCountDown::createOpenScreen() {
	GtkWidget *label;
	GtkWidget *button;
	GtkRcStyle *whiteStyle, *blackStyle;
	GdkColor color;
	
	color.red = 0x4600;
	color.green = 0x8200;
	color.blue = 0xB400;
	
	table = gtk_table_new(5, 2, FALSE);
	gtk_table_set_col_spacings((GtkTable *)table, 40);
	gtk_table_set_row_spacings((GtkTable *)table, 40);

	
	whiteStyle = gtk_rc_style_new();//gtk_widget_get_modifier_style(label);
	whiteStyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(whiteStyle->font_desc, 28 * PANGO_SCALE);
	pango_font_description_set_weight(whiteStyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	whiteStyle->fg[0] = color;
	whiteStyle->color_flags[0] = GTK_RC_FG;
	
	blackStyle = gtk_rc_style_new();//gtk_widget_get_modifier_style(label);
	blackStyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(blackStyle->font_desc, 28 * PANGO_SCALE);
	pango_font_description_set_weight(blackStyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0;
	color.green = 0;
	color.blue = 0;
	blackStyle->fg[0] = color;
	blackStyle->text[0] = color;
	blackStyle->color_flags[0] = (GtkRcFlags)(GTK_RC_FG | GTK_RC_TEXT);

	label = gtk_label_new("Double Bull:");
	
	gtk_widget_modify_style(label, whiteStyle);
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions)(0), 0, 0);
	
	
	button = gtk_button_new();
	gtk_widget_set_size_request(button, 120, 75);
	doubleBullLabel = gtk_label_new("Off");
	gtk_widget_modify_style(doubleBullLabel, blackStyle);
	gtk_misc_set_alignment((GtkMisc *)doubleBullLabel, 0.5, 0.5);

	gtk_container_add((GtkContainer *)button, doubleBullLabel);
	gtk_table_attach (GTK_TABLE (table), button, 1, 2, 2, 3,
			  (GtkAttachOptions)0,
		(GtkAttachOptions)(0), 0, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(onOffToggle), (gpointer)doubleBullLabel);
	button = gtk_button_new();
	
	gtk_widget_set_size_request(button, 220, 75);
	label = gtk_label_new("Start Game");
	gtk_widget_modify_style(label, blackStyle);
	gtk_misc_set_alignment((GtkMisc *)label, 0.5, 0.5);
	
	gtk_container_add((GtkContainer *)button, label);
	
	gtk_table_attach (GTK_TABLE (table), button, 0, 2, 4, 5,
			  (GtkAttachOptions)0,
		(GtkAttachOptions)(0), 0, 0);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(startGame), (gpointer)this);
 pango_font_description_free(whiteStyle->font_desc);
 pango_font_description_free(blackStyle->font_desc);
	return table;
}

void DartCountDown::initScreen() {
	GtkWidget *label;
	GtkWidget *hbox, *vbox;
	std::stringstream sstr;
	GdkColor color;
	
	GtkRcStyle *rcstyle;
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
	gtk_widget_show(smallScore);
	gtk_widget_show(largeScore);
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
	for (int i = 0; i < numberOfPlayers; ++i) {
		gtk_widget_show(userScores[i]);
	}
	
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
	
	lhvbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start((GtkBox *)tophbox, lhvbox, FALSE, FALSE, 0);
	
	midvbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_set_size_request(midvbox, 400, -1);
	gtk_box_pack_start((GtkBox *)tophbox, midvbox, FALSE, FALSE, 0);
	
	rhvbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start((GtkBox *)tophbox, rhvbox, FALSE, FALSE, 0);
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, 0, 30);
	gtk_box_pack_start((GtkBox *)lhvbox, label, FALSE, FALSE, 0);
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, 0, 30);
	gtk_box_pack_start((GtkBox *)rhvbox, label, FALSE, FALSE, 0);
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
	for (int i = 0; i < 1; ++i) {
		sstr.str("");
		sstr << "Player " << i + 1 << " - 0.00";
		playerLabels[i] = gtk_label_new(sstr.str().c_str());
		gtk_widget_set_size_request(playerLabels[i], 200, 30);
		rcstyle = gtk_rc_style_new();//gtk_widget_get_modifier_style(label);
		rcstyle->font_desc = pango_font_description_new();
		pango_font_description_set_size(rcstyle->font_desc, 20 * PANGO_SCALE);
		pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
		color.red = 0xFFFF;
		color.green = 0xFFFF;
		color.blue = 0xFFFF;
		rcstyle->fg[0] = color;
		rcstyle->color_flags[0] = GTK_RC_FG;
		gtk_widget_modify_style(playerLabels[i], rcstyle);
  pango_font_description_free(rcstyle->font_desc);
		// Start of Player box
		
		//gtk_box_pack_start((GtkBox *)vbox, eventBox, FALSE, FALSE, 0);
		sstr.str("");
		sstr << 301;
		playerScoreLabels[i] = gtk_label_new(sstr.str().c_str());
		gtk_widget_set_size_request(playerScoreLabels[i], 200, 100);
		rcstyle = gtk_rc_style_new();
		//rcstyle = gtk_widget_get_style(playerScoreLabels[i]);
		rcstyle->font_desc = pango_font_description_new();
		pango_font_description_set_size(rcstyle->font_desc, 60 * PANGO_SCALE);
		color.red = 0xFFFF;
		color.green = 0xFFFF;
		color.blue = 0xFFFF;
		rcstyle->fg[0] = color;
		rcstyle->color_flags[0] = GTK_RC_FG;
		pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
		gtk_widget_modify_style(playerScoreLabels[i], rcstyle);

		// end of player box
		
		if (i % 2) {
			gtk_box_pack_start((GtkBox *)rhvbox, playerLabels[i], FALSE, FALSE, 0);
			gtk_box_pack_start((GtkBox *)rhvbox, playerScoreLabels[i], FALSE, FALSE, 0);
		}
		else {
			gtk_box_pack_start((GtkBox *)lhvbox, playerLabels[i], FALSE, FALSE, 0);
			gtk_box_pack_start((GtkBox *)lhvbox, playerScoreLabels[i], FALSE, FALSE, 0);
		}
  pango_font_description_free(rcstyle->font_desc);
	}
	
	for (int i = 1; i < 2; ++i) {
		sstr.str("");
		sstr << "Time";
		playerLabels[i] = gtk_label_new(sstr.str().c_str());
		gtk_widget_set_size_request(playerLabels[i], 200, 30);
		rcstyle = gtk_rc_style_new();//gtk_widget_get_modifier_style(label);
		rcstyle->font_desc = pango_font_description_new();
		pango_font_description_set_size(rcstyle->font_desc, 20 * PANGO_SCALE);
		pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
		color.red = 0xFFFF;
		color.green = 0xFFFF;
		color.blue = 0xFFFF;
		rcstyle->fg[0] = color;
		rcstyle->color_flags[0] = GTK_RC_FG;
		gtk_widget_modify_style(playerLabels[i], rcstyle);
  pango_font_description_free(rcstyle->font_desc);
		// Start of Player box
		
		//gtk_box_pack_start((GtkBox *)vbox, eventBox, FALSE, FALSE, 0);
		sstr.str("");
		sstr << "00:00";
		playerScoreLabels[i] = gtk_label_new(sstr.str().c_str());
		gtk_widget_set_size_request(playerScoreLabels[i], 200, 100);
		rcstyle = gtk_rc_style_new();
		//rcstyle = gtk_widget_get_style(playerScoreLabels[i]);
		rcstyle->font_desc = pango_font_description_new();
		pango_font_description_set_size(rcstyle->font_desc, 60 * PANGO_SCALE);
		color.red = 0xFFFF;
		color.green = 0xFFFF;
		color.blue = 0xFFFF;
		rcstyle->fg[0] = color;
		rcstyle->color_flags[0] = GTK_RC_FG;
		pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
		gtk_widget_modify_style(playerScoreLabels[i], rcstyle);

		// end of player box
		
		if (i % 2) {
			gtk_box_pack_start((GtkBox *)rhvbox, playerLabels[i], FALSE, FALSE, 0);
			gtk_box_pack_start((GtkBox *)rhvbox, playerScoreLabels[i], FALSE, FALSE, 0);
		}
		else {
			gtk_box_pack_start((GtkBox *)lhvbox, playerLabels[i], FALSE, FALSE, 0);
			gtk_box_pack_start((GtkBox *)lhvbox, playerScoreLabels[i], FALSE, FALSE, 0);
		}
  pango_font_description_free(rcstyle->font_desc);
	}
	
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, -1, 20);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
	
	playerLabel = gtk_label_new("Player 1 Score");
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 30 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->color_flags[0] = GTK_RC_FG;
	gtk_widget_modify_style(playerLabel, rcstyle);
	gtk_box_pack_start((GtkBox *)midvbox, playerLabel, FALSE, FALSE, 0);
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, -1, 5);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
	
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_set_size_request(hbox, 300, 150);
	
	std::cout << "Line " << __LINE__ << std::endl;
	std::cout.flush();
 pango_font_description_free(rcstyle->font_desc);
	
	sstr.str("");
	sstr << 301;
	currentTotalLabel = gtk_label_new(sstr.str().c_str());
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 80 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->xthickness = 0;
	rcstyle->color_flags[0] = GTK_RC_FG;
	gtk_widget_modify_style(currentTotalLabel, rcstyle);
	gtk_box_pack_start((GtkBox *)hbox, currentTotalLabel, TRUE, FALSE, 0);
 pango_font_description_free(rcstyle->font_desc);
	
	gtk_box_pack_start((GtkBox *)midvbox, hbox, FALSE, FALSE, 0);
	
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, -1, 10);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
	
	
	label = gtk_label_new("Current Score");
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 30 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->color_flags[0] = GTK_RC_FG;
	gtk_widget_modify_style(label, rcstyle);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
 pango_font_description_free(rcstyle->font_desc);
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, -1, 5);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
	
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_set_size_request(hbox, 200, 100);
	
	currentScoreLabel = gtk_label_new("301");
	gtk_widget_set_size_request(currentScoreLabel, 200, 100);
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 60 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->color_flags[0] = GTK_RC_FG;
	gtk_widget_modify_style(currentScoreLabel, rcstyle);
	//gtk_container_add((GtkContainer *)eventBox, currentScoreLabel);
 pango_font_description_free(rcstyle->font_desc);
	//gtk_box_pack_start((GtkBox *)hbox, eventBox, TRUE, FALSE, 0);
	gtk_box_pack_start((GtkBox *)hbox, currentScoreLabel, TRUE, FALSE, 0);
	gtk_box_pack_start((GtkBox *)midvbox, hbox, FALSE, FALSE, 0);
	
	
	label = gtk_label_new(" ");
	gtk_widget_set_size_request(label, -1, 10);
	gtk_box_pack_start((GtkBox *)midvbox, label, FALSE, FALSE, 0);
	
	round = 1;
	sstr.str("");
	sstr << "Round " << std::setw(3) << round;
	roundLabel = gtk_label_new(sstr.str().c_str());
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 30 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->color_flags[0] = GTK_RC_FG;
	gtk_widget_modify_style(roundLabel, rcstyle);
	gtk_box_pack_start((GtkBox *)midvbox, roundLabel, FALSE, FALSE, 0);
 pango_font_description_free(rcstyle->font_desc);
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start((GtkBox *)midvbox, vbox, FALSE, FALSE, 0);
	
	for (int i = 0; i < 3; ++i) {
		
		dartImages[i] = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/dart.gif");
		gtk_box_pack_start((GtkBox *)vbox, dartImages[i], FALSE, FALSE, 2);
		dartLabels[i] = gtk_label_new("");
		rcstyle = gtk_rc_style_new();
		rcstyle->font_desc = pango_font_description_new();
		pango_font_description_set_size(rcstyle->font_desc, 23 * PANGO_SCALE);
		pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
		color.red = 0xFFFF;
		color.green = 0xFFFF;
		color.blue = 0xFFFF;
		rcstyle->fg[0] = color;
		rcstyle->color_flags[0] = GTK_RC_FG;
		gtk_widget_modify_style(dartLabels[i], rcstyle);
		gtk_box_pack_start((GtkBox *)vbox, dartLabels[i], FALSE, FALSE, 2);
  pango_font_description_free(rcstyle->font_desc);
	}
	
	gtk_widget_destroy(table);
	gtk_widget_show_all(tophbox);
	
	for (int i = 0; i < 3; ++i) {
		gtk_widget_hide(dartLabels[i]);
	}
	openSerial();
	
	timer = gtk_timeout_add(100, timeout, this);
	currentPlayer = 0;
	currentDart = 0;
	currentScore = 0;
	
	tz.tz_minuteswest = 0;
	tz.tz_dsttime = 0;
	gettimeofday(&startTime, &tz);
	
}

void DartCountDown::createPPDWindow() {
	GtkWidget *vbox;
	GdkColor color;
	GtkRcStyle *rcstyle;
	
	ppdWindow = gtk_event_box_new();
	
	
	gtk_widget_set_size_request(ppdWindow, 300, 150);
	rcstyle = gtk_rc_style_new();

	rcstyle->bg_pixmap_name[0] = "/usr/local/share/touchmegames/dart01/images/ppdbg.xpm";
	gtk_widget_modify_style(ppdWindow, rcstyle);
	
	gtk_widget_modify_bg(ppdWindow, GTK_STATE_NORMAL, &color);
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add((GtkContainer *)ppdWindow, vbox);
	
	ppdPlayerLabel = gtk_label_new("");
	rcstyle = gtk_rc_style_new();
	rcstyle->font_desc = pango_font_description_new();
	pango_font_description_set_size(rcstyle->font_desc, 40 * PANGO_SCALE);
	pango_font_description_set_weight(rcstyle->font_desc, PANGO_WEIGHT_BOLD);
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	rcstyle->fg[0] = color;
	rcstyle->color_flags[0] = GTK_RC_FG;
	
	gtk_widget_modify_style(ppdPlayerLabel, rcstyle);
	ppdLabel = gtk_label_new("");
	gtk_widget_modify_style(ppdLabel, rcstyle);
	
	gtk_box_pack_start((GtkBox *)vbox, ppdPlayerLabel, FALSE, FALSE, 5);
	gtk_box_pack_start((GtkBox *)vbox, ppdLabel, FALSE, FALSE, 5);
	
	gtk_fixed_put(GTK_FIXED(fixed), ppdWindow, 250, 300);
 pango_font_description_free(rcstyle->font_desc);
}

void DartCountDown::createGameOver1() {
	GtkWidget *label;
	GtkWidget *vbox;
	GtkWidget *button;
	GtkWidget *eventBox;
	GdkColor color;
	GtkStyle *style;
	std::string str;
	int lh(1), rh(2);
	int round;
	
	std::cout << "Winner " << winner << std::endl;
	
	str = gtk_label_get_text((GtkLabel *)playerScoreLabels[1]);
	std::cout << "String: " << str << std::endl;
	highScore = atoi(str.substr(0, 2).c_str());
	highScore += (float)atoi(str.substr(3, 2).c_str()) / 100.0;
	std::cout << "highScore " << highScore << std::endl;
	
	
	gtk_widget_hide(smallScore);
	gtk_widget_hide(largeScore);
	for (int i = 0; i < 8; ++i) {
		gtk_widget_hide(userScores[i]);
	}
	
	round = atoi(gtk_label_get_text((GtkLabel *)roundLabel));
	for (int i = 0; i < numberOfPlayers; ++i) {
		finalScore[i] = 301 - atoi(gtk_label_get_text((GtkLabel *)playerScoreLabels[i]));
	}
	
	gtk_widget_destroy(lhvbox);
	gtk_widget_destroy(rhvbox);
	gtk_widget_destroy(midvbox);
	
	vbox = gtk_vbox_new(FALSE, 0);
	table = gtk_table_new(14, 9, TRUE);
	gtk_box_pack_start((GtkBox *)vbox, table, FALSE, FALSE, 0);
	gtk_table_set_row_spacings((GtkTable *)table, 3);
	gtk_table_set_col_spacings((GtkTable *)table, 3);
	
	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	
	label = gtk_label_new("Winner");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	style = gtk_widget_get_style(label);
	style->fg[0] = color;
	pango_font_description_set_size(style->font_desc, 14 * PANGO_SCALE);
	pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	
	label = gtk_label_new("Time");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	style = gtk_widget_get_style(label);
	style->fg[0] = color;
	pango_font_description_set_size(style->font_desc, 14 * PANGO_SCALE);
	pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
			  
	label = gtk_label_new("Points");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	style = gtk_widget_get_style(label);
	style->fg[0] = color;
	pango_font_description_set_size(style->font_desc, 14 * PANGO_SCALE);
	pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
			  
	label = gtk_label_new("Darts");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	style = gtk_widget_get_style(label);
	style->fg[0] = color;
	pango_font_description_set_size(style->font_desc, 14 * PANGO_SCALE);
	pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("Rounds");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	style = gtk_widget_get_style(label);
	style->fg[0] = color;
	pango_font_description_set_size(style->font_desc, 14 * PANGO_SCALE);
	pango_font_description_set_weight(style->font_desc, PANGO_WEIGHT_BOLD);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	  
			  
	label = gtk_label_new("Doubles");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 6, 7,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("Triples");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 7, 8,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
			  
	label = gtk_label_new("Bulls");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 8, 9,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("PPD");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 9, 10,
			  (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("PPR");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach(GTK_TABLE (table), label, 0, 1, 10, 11,
			 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			  (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("Low Tons");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach(GTK_TABLE (table), label, 0, 1, 12, 13,
			 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			 (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("High Tons");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach(GTK_TABLE (table), label, 0, 1, 13, 14,
			 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			 (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("Ton 80s");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach(GTK_TABLE (table), label, 0, 1, 14, 15,
			 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			 (GtkAttachOptions)(0), 0, 0);
	
	label = gtk_label_new("Max Ton");
	gtk_misc_set_alignment((GtkMisc *)label, 0, 0.5);
	gtk_widget_set_style(label, style);
	gtk_table_attach(GTK_TABLE (table), label, 0, 1, 15, 16,
			 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
			 (GtkAttachOptions)(0), 0, 0);
	
	
	
	for (int i = 0; i < numberOfPlayers; ++i) {
		std::stringstream sstr;
		// Label
		sstr << "Player " << i + 1;
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 0, 1,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << (winner == i ? 1: 0);
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 1, 2,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << std::setw(2) << std::setfill(' ') << (int)highScore << ':' 
				<< std::setw(2) << std::setfill('0') << (int)(highScore * 100.0) % 100;
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 2, 3,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
				 
				 
		sstr.str("");
		sstr << finalScore[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 3, 4,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
				 
		sstr.str("");
		sstr << darts[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 4, 5,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << rounds[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 5, 6,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << doubles[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 6, 7,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << bulls[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 7, 8,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
				 
		sstr.str("");
		sstr << triples[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 8, 9,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		
		sstr << std::setprecision(3) << (float)finalScore[i] / (float)darts[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 9, 10,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << std::setprecision(4) << (float)finalScore[i] / (float)rounds[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 10, 11,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << lowTons[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 12, 13,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << highTons[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 13, 14,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << ton80s[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 14, 15,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		sstr.str("");
		sstr << maxTon[i];
		label = gtk_label_new(sstr.str().c_str());
		gtk_misc_set_alignment((GtkMisc *)label, 1, 0.5);
		gtk_widget_set_style(label, style);
		gtk_table_attach(GTK_TABLE (table), label, lh, rh, 15, 16,
				 (GtkAttachOptions)(GTK_FILL | GTK_EXPAND),
				 (GtkAttachOptions)(0), 0, 0);
		
		
		++rh;
		++lh;
	}
	
	
	eventBox = gtk_event_box_new();
	button = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/exitgame.gif");
	gtk_event_box_set_visible_window((GtkEventBox *)eventBox, FALSE);
	gtk_widget_add_events(eventBox, GDK_BUTTON_PRESS_MASK);
	g_signal_connect(G_OBJECT(eventBox), 
			 "button-press-event",
			 G_CALLBACK(exitGame), (gpointer)this);
	
	gtk_container_add((GtkContainer *)eventBox, button);
	gtk_box_pack_start((GtkBox *)vbox, eventBox, FALSE, FALSE, 50);
	gtk_box_pack_start((GtkBox *)tophbox, vbox, FALSE, FALSE, 0);
	gtk_widget_show_all(vbox);
}

void DartCountDown::createLowTon() {
	GtkWidget *image;
	
	image = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/lowton.gif");
	
	lowTon = gtk_event_box_new();
	gtk_event_box_set_visible_window((GtkEventBox *)lowTon, FALSE);
	gtk_container_add((GtkContainer *)lowTon, image);
	gtk_fixed_put(GTK_FIXED(fixed), lowTon, 300, -300);
	gtk_widget_show_all(lowTon);
	lowTonY = -300;
}

void DartCountDown::createHighTon() {
	GtkWidget *image;
	
	image = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/highton.gif");
	
	highTon = gtk_event_box_new();
	gtk_event_box_set_visible_window((GtkEventBox *)highTon, FALSE);
	gtk_container_add((GtkContainer *)highTon, image);
	gtk_fixed_put(GTK_FIXED(fixed), highTon, 200, -400);
	gtk_widget_show_all(highTon);
	highTonY = -400;
}

void DartCountDown::createTon80() {
	GtkWidget *image;
	
	image = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/ton80.gif");
	
	ton80 = gtk_event_box_new();
	gtk_event_box_set_visible_window((GtkEventBox *)ton80, FALSE);
	gtk_container_add((GtkContainer *)ton80, image);
	gtk_fixed_put(GTK_FIXED(fixed), ton80, 100, -600);
	gtk_widget_show_all(ton80);
	ton80Y = -400;
}

GtkWidget *DartCountDown::init() {
	GAME_PTR::loadScores(scores);
	numberOfPlayers = 1;
	for (int i = 0; i < 10; ++i) {
		std::cout << scores[i].fscore << std::endl;
	}
	for (int i = 0; i < 8; ++i) {
		darts[i] = 0;
		finalScore[i] = 0;
		rounds[i] = 0;
		doubles[i] = 0;
		triples[i] = 0;
		bulls[i] = 0;
		lowTons[i] = 0;
		highTons[i] = 0;
		ton80s[i] = 0;
		maxTon[i] = 0;
	}
	winner = -1;
	beginningScore = 0; // for when the player busts

	fixed = gtk_fixed_new();
	gtk_widget_ref(fixed);
	
	tophbox = gtk_hbox_new(FALSE, 0);
	
	createPPDWindow();
	
	// create the background images
	for (int i = 0; i < 8; ++i) {
		userScores[i] = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/smallscorebg.xpm");
		gtk_fixed_put((GtkFixed *)fixed, userScores[i], (i & 1 ? 600 : 0), 60 + ((i & 1 ? i / 2: i) * 130));
	}
	smallScore = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/smallscorebg.xpm");
	gtk_fixed_put((GtkFixed *)fixed, smallScore, 300, 290);
	largeScore = gtk_image_new_from_file("/usr/local/share/touchmegames/dart01/images/largescorebg.xpm");
	gtk_fixed_put((GtkFixed *)fixed, largeScore, 250, 75);
	
	gtk_box_pack_start((GtkBox *)tophbox, createOpenScreen(), TRUE, FALSE, 5);
	
	gtk_fixed_put(GTK_FIXED(fixed), tophbox, 0, 0);
	
	gtk_widget_show_all(fixed);
	gtk_widget_hide_all(ppdWindow);
	gtk_widget_hide(smallScore);
	gtk_widget_hide(largeScore);
	for (int i = 0; i < 8; ++i) {
		gtk_widget_hide(userScores[i]);
	}
	
	return fixed;
}

void DartCountDown::openSerial() {
	if (!dartBoard.init("/dev/ttyS0")) {
		std::cout << "No init" << std::endl;
	}
	if (!dartBoard.dartBoardPresent()) {
		// we gots ourselves some trouble.
	}
}
